/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package heatmap;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import javax.imageio.ImageIO;

/**
 * This class can create a heatmap from a given list of points.
 * The output will be a picture visualising the occurences of points.
 * <p>
 * CIRCLEPIC (an image of a circle, from black to transparent)
 * <p>
 * and
 * <p>
 * SPECTRUMPIC (a color gradiant
 *  where the color that represents "most" is at the bottom)
 * <p>
 * must be in the user.dir in:
 * <p>
 * heatmap/bolillaT.png
 * <p>
 * and
 * <p>
 * heatmap/colors.png
 *
 * @author sam
 */
public class HeatMapNew {

    /** half of the size of the circle picture. */
    private static final int HALFCIRCLEPICSIZE = 32;
    /** path to picture of circle which gets more transparent to the outside. */
    private static final String CIRCLEPIC = System.getProperty("user.dir")
            + File.separator + "heatmap" + File.separator + "bolillaT.png";
    private static final String SPECTRUMPIC = System.getProperty("user.dir")
            + "/heatmap/colors.png";
    /** map to collect and sort points. */
    private Map<Integer, List<Point>> map;
    /** maximum occurance of the same coordinates. */
    private int maxOccurance = 1;
    /** maximal given x value. */
    private int maxXValue;
    /** maximal given y value. */
    private int maxYValue;
    /** name of file over which the heatmap will be laid. */
    private String lvlMap;
    /** name of file to save heatmap to. */
    private String outputFile;

    /**
     * constructs new instance of HeatMap from given list of points.
     * This may take a while, as the points are being sorted.
     *
     * @param points th list of points
     * @param output name of file to store created heatmap in
     * @param lvlMap name of file to lay heatmap over
     */
    public HeatMapNew(List<Point> points, String output, String lvlMap) {
        outputFile = output;
        this.lvlMap = lvlMap;
        initMap(points);
    }

    /**
     * initiate map.
     * counts and sorts points and figures out max x and y values as well
     * as the maximal amount of points with same coordinates.
     * max x and y values will be used for the size of the heatmap.
     *
     * @param points list of points
     */
    private void initMap(List<Point> points) {
        map = new HashMap<Integer, List<Point>>();
        BufferedImage mapPic = loadImage(lvlMap);
        maxXValue = mapPic.getWidth();
        maxYValue = mapPic.getHeight();

        int pointSize = points.size();
        for (int i = 0; i < pointSize; i++) {
            Point point = points.get(i);
            // add point to right list.
            int hash = getkey(point);
            if (map.containsKey(hash)) {
                List<Point> thisList = map.get(hash);
                thisList.add(point);
                if (thisList.size() > maxOccurance) {
                    maxOccurance = thisList.size();
                }
                // if list did not exist, create new one and add point.
            } else {
                List<Point> newList = new LinkedList<Point>();
                newList.add(point);
                map.put(hash, newList);
            }
        }
    }

    /**
     * creates the heatmap.
     *
     * @param multiplier calculated opacity of every point will be
     * multiplied by this value. This leads to a HeatMap that is easier to read,
     * especially when there are not too many points or the points are too
     * spread out. Pass 1.0f for original.
     */
    public void createHeatMap(float multiplier) {

        BufferedImage circle = loadImage(CIRCLEPIC);
        BufferedImage heatMap = new BufferedImage(maxXValue, maxYValue, 6);
        makeTransparentToColor(heatMap, Color.white);

        Iterator<List<Point>> iterator = map.values().iterator();
        while (iterator.hasNext()) {
            List<Point> currentPoints = iterator.next();

            // calculate opaqueness
            // based on number of occurences of current point
            float opaque = currentPoints.size()
                    / (float) maxOccurance;

            // adjust opacity so the heatmap is easier to read
            opaque = opaque * multiplier;
            if (opaque > 1) {
                opaque = 1;
            }

            Point currentPoint = currentPoints.get(0);

            // draw a circle which gets transparent from middle to outside
            // (which opaqueness is set to "opaque")
            // at the position specified by the center of the currentPoint
            addImage(heatMap, circle, opaque,
                    (currentPoint.x - HALFCIRCLEPICSIZE),
                    (currentPoint.y - HALFCIRCLEPICSIZE));
        }
        print("done adding points.");

        // negate the image
        heatMap = negateImage(heatMap);

        // remap black/white with color spectrum from white over red, orange,
        // yellow, green to blue
        remap(heatMap);

        // blend image over lvlMap at opacity 40%
        BufferedImage output = loadImage(lvlMap);
        addImage(output, heatMap, 0.4f);

        // save image
        saveImage(output, outputFile);
        print("done creating heatmap.");
    }

    /**
     * remaps black and white picture with colors.
     * It uses the colors from SPECTRUMPIC. The whiter a pixel is, the more it
     * will get a color from the bottom of it. Black will stay black.
     *
     * @param heatMapBW black and white heat map
     */
    private void remap(BufferedImage heatMapBW) {
        BufferedImage colorGradiant = loadImage(SPECTRUMPIC);

        for (int i = 0; i < heatMapBW.getWidth(); i++) {
            for (int j = 0; j < heatMapBW.getHeight(); j++) {
                // get heatMapBW color values:
                Color c = new Color(heatMapBW.getRGB(i, j));
                float[] floats = new float[4];
                c.getRGBColorComponents(floats);

                // calculate multiplier to be applied to height of gradiant.
                float multiplier = 1.0f;
                multiplier *= (c.getBlue() / 255f);
                multiplier *= (c.getGreen() / 255f);
                multiplier *= (c.getRed() / 255f);

                // apply multiplier
                int y = (int) (multiplier * colorGradiant.getHeight());
                if (y >= colorGradiant.getHeight()) {
                    y = colorGradiant.getHeight() - 1;
                }

                // remap values
                // calculate new value based on whitenes of heatMap
                // (the whiter, the more a color from the top of colorGradiant
                // will be chosen.
                int mapedRGB = colorGradiant.getRGB(0, y);
                // set new value
                heatMapBW.setRGB(i, j, mapedRGB);
            }
        }
    }

    /**
     * saves the image in the provided buffer to the destination.
     *
     * @param buff buffer to be saved
     * @param dest destination to save at
     */
    private void saveImage(BufferedImage buff, String dest) {
        try {
            File outputfile = new File(dest);
            ImageIO.write(buff, "png", outputfile);
        } catch (IOException e) {
            print("error saving the image: " + dest + ": " + e);
        }
    }

    /**
     * returns a BufferedImage from the Image provided.
     *
     * @param ref path to image
     * @return loaded image
     */
    private BufferedImage loadImage(String ref) {
        BufferedImage b1 = null;
        try {
            b1 = ImageIO.read(new File(ref));
        } catch (IOException e) {
            System.out.println("error loading the image: " + ref + " : " + e);
        }
        return b1;
    }

    /**
     * returns a negated version of this image.
     *
     * @param img buffer to negate
     * @return negated buffer
     */
    private BufferedImage negateImage(BufferedImage img) {
        Color col;
        for (int x = 0; x < img.getWidth(); x++) { //width
            for (int y = 0; y < img.getHeight(); y++) { //height

                //get RGBA data for the specific pixel
                int rGBA = img.getRGB(x, y);

                //get the color data of the specific pixel
                col = new Color(rGBA, true);

                col = new Color(Math.abs(col.getRed() - 255),
                        Math.abs(col.getGreen() - 255),
                        Math.abs(col.getBlue() - 255)); //Swaps values
                //i.e. 255, 255, 255 (white)
                //becomes 0, 0, 0 (black)

                //set the pixel to the altered colors
                img.setRGB(x, y, col.getRGB());
            }
        }
        return img;
    }

    /**
     * changes the transparent parts of the provided image
     * to the provided color.
     *
     * @param buff buffer
     * @param color color to replace with transparency
     */
    private void makeTransparentToColor(BufferedImage buff, Color color) {
        Graphics2D g2 = buff.createGraphics();
        g2.drawImage(buff, 0, 0, color, null);
        g2.dispose();
    }

    /**
     * changes the opacity of the image.
     *
     * @param buff1 buffer to change opacity
     * @param opaque new opacity
     */
    private void makeTransparent(BufferedImage buff1, float opaque) {
        Graphics2D g2d = buff1.createGraphics();
        g2d.setComposite(
                AlphaComposite.getInstance(AlphaComposite.SRC, opaque));
        g2d.drawImage(buff1, 0, 0, null);
        g2d.dispose();
    }

    /**
     * prints the contents of buff2 on buff1 with the given opaque value
     * starting at position 0, 0.
     *
     * @param buff1 buffer
     * @param buff2 buffer to add to buff1
     * @param opaque opacity
     */
    private void addImage(
            BufferedImage buff1, BufferedImage buff2, float opaque) {
        addImage(buff1, buff2, opaque, 0, 0);
    }

    /**
     * prints the contents of buff2 on buff1 with the given opaque value.
     *
     * @param buff1 buffer
     * @param buff2 buffer
     * @param opaque how opaque the second buffer should be drawn
     * @param x x position where the second buffer should be drawn
     * @param y y position where the second buffer should be drawn
     */
    private void addImage(BufferedImage buff1, BufferedImage buff2,
            float opaque, int x, int y) {
        Graphics2D g2d = buff1.createGraphics();
        g2d.setComposite(
                AlphaComposite.getInstance(AlphaComposite.SRC_OVER, opaque));
        g2d.drawImage(buff2, x, y, null);
        g2d.dispose();
    }

    /**
     * returns a hash calculated by the given point.
     *
     * @param p a point
     * @return hash value
     */
    private int getkey(Point p) {
        return ((p.x << 19) | (p.y << 7));
    }

    /**
     * prints string to sto.
     *
     * @param s string to print
     */
    private void print(String s) {
        System.out.println(s);
    }
}
