/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.util.ArrayList;
import java.util.List;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.ImageBuffer;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Circle;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.tiled.TiledMap;

/**
 * This class loads and stores the information given in a .tmx file
 * created with the program Tiled ({@link http://www.mapeditor.org/ }).
 * <p>
 * It checks the map for tiles with the Layer-ID 0 and assumes
 * that they are not passable, so it creates <code>CollisionBlock</code> with
 * the tile size and position provided by the map.
 * Tiles with tile-ID 1 will be unpassable,
 * Tiles with tile-ID 5 will kill the player instantly,
 * <p>
 * It ignores all other tiles as decoration. Meaning they do not affect
 * game play but are printed to the screen.
 * <p>
 * It can also load the player at the position and the size defined in
 * group-ID 0 and object-ID 0.
 * <p>
 * Gear will be loaded from group-ID 1 and object-ID 0 to the maximal object-ID.
 * If the property "draw" is set to "false" the gear will not be rendered.
 * <p>
 * level complete tiles will be loaded from the layers with ID 2 and 3 and any
 * tile-ID above 0. layer-ID 2 will be used as normal finish, layer-ID 3 as
 * hidden finish (if existing).
 * <p>
 * Water tiles will be loaded from layer-ID 4.
 * <p>
 * Canons will be loaded from group-ID 2.
 * They must contain the following preferences:
 *  fr: fireRate
 *  fd: fireDirection
 *  bs: canonBallSpeed
 * <p>
 * The map property "bg" may be set to the background picture which should be
 * used. If none is given, the background will be black.
 * <p>
 * For objects, pictures will not be loaded and must
 * therefore be hardcoded here
 * (tiled QT does not seem to support it so i left it out for
 * compability reasons).
 * // TODO change that (image hardcoded)
 *
 * @see CollisionBlock
 * @see Player
 *
 * @author sam
 */
public class BlockMap {

    private static final int COLLISIONLAYERID = 0;
    private static final int COLLISIONTILEID = 1;
    private static final int KILLTILEID = 5; // same layer id as collision
    private static final int LVLCOMPLETE1LAYERID = 2;
    private static final int LVLCOMPLETE2LAYERID = 3;
    private static final int WATERLAYERID = 4;
    private static final int PLAYERGROUPID = 0;
    private static final int PLAYERID = 0;
    private static final int GEARGROUPID = 1;
    private static final int CANONGROUPID = 2;
    //private static final int MOVINGBLOCKSGROUPID = 2;
    private static final String IMAGEPATH = System.getProperty("user.dir")
            + "/data/";
    private static final String TRANSPARENTIMG = "transparent.png";
    private TiledMap tiledMap;
    private List<CollisionBlock> collisionBlocks;
    private List<CollisionBlock> killBlocks;
    private List<CollisionBlock> lvlEndBlocks1;
    private List<CollisionBlock> lvlEndBlocks2;
    private List<CollisionBlock> waterBlocks;
    private String backgroundPic; // in same dir as lvl!

    /**
     * constructs new BlockMap from given file.
     * The tiled map is loaded as well as the collision blocks.
     *
     * @param pathToMap name (and path) of map file
     *
     * @throws SlickException SlickException
     */
    public BlockMap(String pathToMap) throws Exception {

        // TODO this error handling is terrible. make it better
        try {
            tiledMap = new TiledMap(pathToMap);
        } catch (Exception se) {
            throw new Exception("BlockMap.BlockMap(): could not load tiledmap: "
                    + pathToMap);
        }

        try {
            backgroundPic = tiledMap.getMapProperty("bg", TRANSPARENTIMG);
        } catch (Exception e) {
            backgroundPic = TRANSPARENTIMG;
            GeneralNumbersAndFunctions.logError(
                    this.getClass().getCanonicalName()
                    + ": could not load MapProperty \"bg\": " + e);
        }

        collisionBlocks = new ArrayList<CollisionBlock>();
        killBlocks = new ArrayList<CollisionBlock>();
        lvlEndBlocks1 = new ArrayList<CollisionBlock>();
        lvlEndBlocks2 = new ArrayList<CollisionBlock>();
        waterBlocks = new ArrayList<CollisionBlock>();

        int tileHeight = tiledMap.getTileHeight();
        int tileWidth = tiledMap.getTileWidth();
        int mapHeight = tiledMap.getHeight();
        int mapWidth = tiledMap.getWidth();
        for (int i = 0; i < mapWidth; i++) {
            for (int j = 0; j < mapHeight; j++) {
                // init collision and kill blocks
                int tileID = tiledMap.getTileId(i, j, COLLISIONLAYERID);
                if (tileID == COLLISIONTILEID) {
                    collisionBlocks.add(new CollisionBlock(i * tileWidth,
                            j * tileHeight, tileWidth, tileHeight));
                } else if (tileID == KILLTILEID) {
                    killBlocks.add(new CollisionBlock(i * tileWidth,
                            j * tileHeight, tileWidth, tileHeight));
                }

                // init level finished blocks
                tileID = tiledMap.getTileId(i, j, LVLCOMPLETE1LAYERID);
                if (tileID > 0) {
                    lvlEndBlocks1.add(new CollisionBlock(i * tileWidth,
                            j * tileHeight, tileWidth, tileHeight));
                }
                tileID = tiledMap.getTileId(i, j, LVLCOMPLETE2LAYERID);
                if (tileID > 0) {
                    lvlEndBlocks2.add(new CollisionBlock(i * tileWidth,
                            j * tileHeight, tileWidth, tileHeight));
                }

                try {
                    tileID = tiledMap.getTileId(i, j, WATERLAYERID);
                    if (tileID > 0) {
                        waterBlocks.add(new CollisionBlock(i * tileWidth,
                                j * tileHeight, tileWidth, tileHeight));
                    }
                } catch (Exception e) {
                    // no water layer present. dont do anything.
                }
            }
        }

        // set images of kill and collision blocks to transparent.png so
        // they will not be rendered
        try {
            tiledMap.findTileSet(COLLISIONTILEID).setTileSetImage(
                    new Image(IMAGEPATH + TRANSPARENTIMG));
            tiledMap.findTileSet(KILLTILEID).setTileSetImage(
                    new Image(IMAGEPATH + TRANSPARENTIMG));
        } catch (SlickException e) {
            GeneralNumbersAndFunctions.logError(
                    this.getClass().getCanonicalName() + ": could not load "
                    + IMAGEPATH + "transparent.png: " + e);
        }
    }

    /**
     * returns the background picture to be used for this map.
     *
     * @return backgroundPic
     */
    public String getBackgroundPic() {
        return backgroundPic;
    }

    /**
     * returns the tiledMap.
     *
     * @return tiledMap
     */
    public TiledMap getTiledMap() {
        return tiledMap;
    }

    // TODO refactor three load methods. lots of dublicate code.
    /**
     * creates and returns a new <code>Player</code>.
     * <p>
     * The player is created with the size and position as defined
     * in the map file provided to the <code>BlockMap</code> class.
     * The collision form is a <code>Circle</code> at half the object width
     * and half the object height with a radius of the maximum of half the
     * object height/width.
     *
     * @return Player Player
     * @throws SlickException SlickException
     */
    public Player loadPlayer(String name) throws SlickException {
        Circle playerCicle = new Circle(
                tiledMap.getObjectWidth(PLAYERGROUPID, PLAYERID) / 2f,
                tiledMap.getObjectHeight(PLAYERGROUPID, PLAYERID) / 2f,
                Math.max(tiledMap.getObjectWidth(PLAYERGROUPID, PLAYERID) / 2f,
                tiledMap.getObjectHeight(PLAYERGROUPID, PLAYERID) / 2f));
        // Image img = new Image(IMAGEPATH + tiledMap.getObjectImage(
        //      PLAYERGROUPID, PLAYERID));
        Image img;
        try {
            img = new Image(IMAGEPATH + "stickman.png");
        } catch (Exception ex) {
            GeneralNumbersAndFunctions.logError("could not load " + IMAGEPATH
                    + "stickman.png");
            ImageBuffer imageBuffer = new ImageBuffer(40, 40);
            for (int i = 0; i < 20; i++) {
                for (int j = 0; j < 20; j++) {
                    imageBuffer.setRGBA(i, j, 150, 150, 150, 150);
                }
            }
            img = new Image(imageBuffer);
        }

        Player player = new Player(
                name,
                img.getScaledCopy(tiledMap.getObjectWidth(PLAYERGROUPID,
                PLAYERID),
                tiledMap.getObjectHeight(PLAYERGROUPID, PLAYERID)),
                new Vector2f(tiledMap.getObjectX(PLAYERGROUPID, PLAYERID),
                tiledMap.getObjectY(PLAYERGROUPID, PLAYERID)),
                playerCicle,
                this);
        return player;
    }

    /**
     * creates and returns a new <code>Collectable</code> list.
     * <p>
     * The collectable is created with the size and position as defined
     * in the map file provided to the <code>BlockMap</code> class.
     * The collision form is a <code>Circle</code> at half the object width
     * and half the object height with a radius of the maximum of half the
     * object height/width.
     *
     * @return Collectable list
     */
    public List<Collectable> loadGear() {
        List gearList = new ArrayList();
        if (tiledMap.getObjectCount(GEARGROUPID) <= 0) {
            // no gears, return empty list
            return gearList;
        }
        // Image img = new Image(IMAGEPATH +
        //              tiledMap.getObjectImage(GEARGROUPID, 0));
        Image img;
        try {
            img = new Image(IMAGEPATH + "gearsmall.png");
        } catch (Exception ex) {
            GeneralNumbersAndFunctions.logError("could not load " + IMAGEPATH
                    + "gearsmall.png");
            ImageBuffer imageBuffer = new ImageBuffer(20, 20);
            for (int i = 0; i < 20; i++) {
                for (int j = 0; j < 20; j++) {
                    imageBuffer.setRGBA(i, j, 150, 150, 150, 150);
                }
            }
            img = new Image(imageBuffer);
        }
        Circle gearCicle = new Circle(
                tiledMap.getObjectWidth(GEARGROUPID, 0) / 2f,
                tiledMap.getObjectHeight(GEARGROUPID, 0) / 2f,
                Math.max(tiledMap.getObjectWidth(GEARGROUPID, 0) / 2f,
                tiledMap.getObjectHeight(GEARGROUPID, 0) / 2f));


        for (int i = 0; i < tiledMap.getObjectCount(GEARGROUPID); i++) {
            boolean drawCollectable = true;
            try {
                String drawProperty = tiledMap.getObjectProperty(GEARGROUPID, i,
                        "draw", "true");
                if ("false".equals(drawProperty)) {
                    drawCollectable = false;
                }
            } catch (Exception e) {
                drawCollectable = true;
            }

            Collectable gear = new Collectable(
                    "gear",
                    // FIXME scaled copy is not displayed
                    // (and move scaling outside loop)
                    // img.getScaledCopy(tiledMap.getObjectWidth
                    //      (GEARGROUPID, i)),
                    img,
                    new Vector2f(tiledMap.getObjectX(GEARGROUPID, i),
                    tiledMap.getObjectY(GEARGROUPID, i)),
                    gearCicle,
                    drawCollectable);
            gearList.add(gear);
        }
        return gearList;
    }

    public List<Canon> loadCanons() {
        List canonList = new ArrayList();
        if (tiledMap.getObjectCount(CANONGROUPID) <= 0) {
            // no canons, return empty list
            return canonList;
        }

        Circle canonCicle = new Circle(
                tiledMap.getObjectWidth(CANONGROUPID, 0) / 2f,
                tiledMap.getObjectHeight(CANONGROUPID, 0) / 2f,
                Math.max(tiledMap.getObjectWidth(CANONGROUPID, 0) / 2f,
                tiledMap.getObjectHeight(CANONGROUPID, 0) / 2f));


        for (int i = 0; i < tiledMap.getObjectCount(CANONGROUPID); i++) {
            // load canon image.
            // something seems to be wrong with the getScaled method.
            // it does also change the settings of the original.
            // so, we need a new image for every canon.
            Image img;
            try {
                img = new Image(IMAGEPATH + "tiles/mine/canon.png");
            } catch (Exception ex) {
                GeneralNumbersAndFunctions.logError("could not load " + IMAGEPATH
                        + "tiles/mine/canon.png");
                ImageBuffer imageBuffer = new ImageBuffer(20, 20);
                for (int n = 0; n < 20; n++) {
                    for (int j = 0; j < 20; j++) {
                        imageBuffer.setRGBA(n, j, 150, 150, 150, 150);
                    }
                }
                img = imageBuffer.getImage();
            }
            float scale = tiledMap.getObjectWidth(CANONGROUPID, 0) / ((float) img.getWidth());
            img = img.getScaledCopy(scale);

            Image ballImage;
            try {
                ballImage = new Image(IMAGEPATH + "tiles/mine/canonBall.png");
            } catch (Exception ex) {
                GeneralNumbersAndFunctions.logError("could not load " + IMAGEPATH
                        + "tiles/mine/canonBall.png");
                ImageBuffer imageBuffer = new ImageBuffer(20, 20);
                for (int n = 0; n < 20; n++) {
                    for (int j = 0; j < 20; j++) {
                        imageBuffer.setRGBA(n, j, 150, 150, 150, 150);
                    }
                }
                ballImage = new Image(imageBuffer);
            }

            ballImage = ballImage.getScaledCopy(scale);

            int fireRate = -1;
            int fireDirection = 3;
            int canonBallSpeed = 0;
            try {
                String fireRateProperty = tiledMap.getObjectProperty(CANONGROUPID, i,
                        "fr", "-1");
                fireRate = Integer.valueOf(fireRateProperty);

                String fireDirectionProperty = tiledMap.getObjectProperty(CANONGROUPID, i,
                        "fd", "3");
                fireDirection = Integer.valueOf(fireDirectionProperty);

                String canonBallSpeedProperty = tiledMap.getObjectProperty(CANONGROUPID, i,
                        "bs", "0");
                canonBallSpeed = Integer.valueOf(canonBallSpeedProperty);

            } catch (Exception e) {
                GeneralNumbersAndFunctions.logError(
                        "failed to load at least one canonBall property: " + e);
            }

            if (fireDirection == 1) {
                img = img.getFlippedCopy(true, false);
            } else if (fireDirection == 3) {
                img = img.getScaledCopy(1);
            } else if (fireDirection == 2) {
                img.setRotation(-90);
            } else if (fireDirection == 4) {
                img.setRotation(90);
            }

            Canon canon = new Canon(
                    "canon",
                    img,
                    ballImage,
                    new Vector2f(tiledMap.getObjectX(CANONGROUPID, i),
                    tiledMap.getObjectY(CANONGROUPID, i)),
                    canonCicle,
                    fireRate, fireDirection, canonBallSpeed);

            canonList.add(canon);
        }
        return canonList;
    }

    /**
     * checks if the player is at the finish of this map.
     * <p>
     * returns 1 if the player is at the normal finish line,
     * 2 if the player is at a hidden finish line,
     * and 0 if the player did not finish yet.
     *
     * @param obj obj
     * @return is complete?
     */
    public int isLevelComplete(ICollidableObject obj) {
        if (isCollidingWith(obj, lvlEndBlocks1)) {
            return 1;
        } else if (isCollidingWith(obj, lvlEndBlocks2)) {
            return 2;
        } else {
            return 0;
        }
    }

    /**
     * returns true, if the given object is colliding with any of the
     * CollisionBlocks in the given list.
     *
     * @param obj obj
     * @param type type
     * @return is colliding?
     */
    private boolean isCollidingWith(ICollidableObject obj,
            List<CollisionBlock> type) {
        Shape collisionShape = obj.getCollisionShape();
        float objCenterX = collisionShape.getCenterX();
        float objCenterY = collisionShape.getCenterY();

        for (CollisionBlock collisionBlock : type) {
            // first, check if block is close to obj (fast)
            if (collisionBlock.isCloseTo(objCenterX, objCenterY)) {
                // if so, check if they are colliding (slower)
                if (collisionBlock.isCollidingWith(obj)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * checks whether or not the passed object is colliding with any of the
     * collision blocks.
     *
     * @param obj obj
     * @return is colliding?
     */
    public boolean isColliding(ICollidableObject obj) {
        return isCollidingWith(obj, collisionBlocks);
    }

    /**
     * checks whether or not the passed object is colliding with any of the
     * kill blocks.
     *
     * @param obj obj
     * @return on kill block?
     */
    public boolean isOnKillBlock(ICollidableObject obj) {
        return isCollidingWith(obj, killBlocks);
    }

    // TODO speed this up (there are too many waterblocks)
    public boolean isOnWaterBlock(ICollidableObject obj) {
        return isCollidingWith(obj, waterBlocks);
    }

    /**
     * returns true, if the passed object is closely above a collision block.
     * In this case, closely means within 2 pixel of it
     * with its highest y value.
     *
     * @param obj obj
     * @return return
     */
    public boolean isRightAboveCollisionBlock(ICollidableObject obj) {
        Shape collisionShape = obj.getCollisionShape();
        return containsPoints(collisionShape.getMinX(),
                collisionShape.getCenterX(),
                collisionShape.getMaxX(),
                collisionShape.getMaxY() + 2);
    }

    /**
     * returns true, if the passed object is closely below a collision block.
     * In this case, closely means within 2 pixel of it with its lowest y value.
     *
     * @param obj obj
     * @return return
     */
    public boolean isRightBelowCollisionBlock(ICollidableObject obj) {
        Shape collisionShape = obj.getCollisionShape();
        return containsPoints(collisionShape.getMinX(),
                collisionShape.getCenterX(),
                collisionShape.getMaxX(),
                collisionShape.getMinY() - 2);
    }

    /**
     * returns true, if any of the given three x values and the y value is
     * within any of the collision blocks of this map.
     *
     * @param minX minX
     * @param centerX centerX
     * @param maxX maxX
     * @param y y
     * @return return
     */
    private boolean containsPoints(
            float minX, float centerX, float maxX, float y) {
        for (CollisionBlock collisionBlock : collisionBlocks) {
            Shape collisionShape = collisionBlock.getCollisionShape();
            if ((collisionShape.contains(minX, y)
                    || collisionShape.contains(maxX, y)
                    || collisionShape.contains(centerX, y))) {
                return true;
            }
        }
        return false;

    }

    /**
     * returns true, if the passed object is closely right of a collision block.
     * In this case, closely means within 2 pixel of it with its lowest x value.
     *
     * @param obj obj
     * @return return
     */
    public boolean isRightRightOfCollisionBlock(ICollidableObject obj) {
        float objCenterY = obj.getCollisionShape().getCenterY();
        float objLoestX = obj.getCollisionShape().getMinX() - 2;
        for (CollisionBlock collisionBlock : collisionBlocks) {
            if (collisionBlock.getCollisionShape().contains(
                    objLoestX, objCenterY)) {
                return true;
            }
        }
        return false;
    }

    /**
     * returns true, if the passed object is closely left of a collision block.
     * In this case, closely means within 2 pixel of it with its lowest x value.
     *
     * @param obj obj
     * @return return
     */
    public boolean isRightLeftOfCollisionBlock(ICollidableObject obj) {
        float objCenterY = obj.getCollisionShape().getCenterY();
        float objHighestX = obj.getCollisionShape().getMaxX() + 2;
        for (CollisionBlock collisionBlock : collisionBlocks) {
            if (collisionBlock.getCollisionShape().contains(
                    objHighestX, objCenterY)) {
                return true;
            }
        }
        return false;
    }

    /**
     * draws collision shapes of CollisionBlocks to provided Graphic.
     *
     * @param g g
     */
    public void renderCollisionShapes(Graphics g) {
        for (CollisionBlock collisionBlock : collisionBlocks) {
            collisionBlock.drawCollisionShape(g);
        }
    }
}