/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * This class is responcible for holding information about open/closed levels.
 * <p>
 * Additionally, this class loads the level file names as well as their
 * display name.
 * <p>
 * IMPORTANT:
 * All levels files must be in the directory specified in
 * GeneralNumberAndFunctions.java
 * and they must be named in the following manner:
 * lvl"lvlNumber""Level.Name".tmx
 *
 * @author sam
 */
public class SettingsLevel {

    private boolean[] levelOpen;
    private List<String> displayNames;
    private List<String> names;
    private Settings settings;

    /**
     * constructs new SettingsLevel.
     * <p>
     * Also loads information about level files from level directory.
     *
     * @param settings settings
     */
    public SettingsLevel(Settings settings) {
        this.settings = settings;

        displayNames = new ArrayList<String>();
        names = new ArrayList<String>();

        // look up how many levels there are
        int amountLevels = 0;
        File dir = new File(GeneralNumbersAndFunctions.LEVELPATH);
        String[] children = dir.list();
        java.util.Arrays.sort(children);
        if (children != null) {
            for (int i = 0; i < children.length; i++) {
                // get name of file or directory
                String filename = children[i];
                if (filename.startsWith("lvl") && filename.endsWith(".tmx")) {
                    amountLevels++;
                    names.add(filename);
                    String levelName = filename.replaceFirst("lvl", "");
                    // level name may not have digits
                    levelName = levelName.replaceAll("\\d", "");
                    // remove file extention
                    levelName = levelName.replaceAll("\\.tmx", "");
                    // prepent all capital leters with a space
                    levelName = levelName.replaceAll("\\.", " ");
                    displayNames.add(levelName);

                }
            }
        }

        // open the levels that should be open
        levelOpen = new boolean[amountLevels];
        levelOpen[0] = true;
        for (int i = 1; i < settings.getOpenLevels() && i < amountLevels; i++) {
            levelOpen[i] = true;
        }
    }

    /**
     * returns whether or not the given level has been opened.
     *
     * @param i level
     * @return is level open?
     */
    public boolean isLevelOpen(int i) {
        if (Settings.debug) {
            return true;
        }
        if (i > 0 && i <= getNumberOfLevels()) {
            return levelOpen[i - 1];
        }
        return false;
    }

    /**
     * returns the name of the level.
     * A level file should be named as following:
     * lvl"lvlNumber""Level.Name".tmx
     * "Level.Name" with the "."s removed will be returned.
     *
     * @param i level
     * @return display name
     */
    public String getLevelDisplayName(int i) {
        if (i > 0 && i <= getNumberOfLevels()) {
            return displayNames.get(i - 1);
        }
        return "";
    }

    /**
     * returns the level file name belonging to the passed level.
     *
     * @param i level
     * @return file name
     */
    public String getLevelName(int i) {
        if (i > 0 && i <= getNumberOfLevels()) {
            return names.get(i - 1);
        }
        return "";
    }

    /**
     * gives access to the given level.
     * If i is bigger than one nothing happens (you cannot open a negativ level
     * and level one is always open). If it is bigger that the number of levels
     * it will do nothing as well.
     * If level i is opened, i will be saved in the settings as number of level
     * opened.
     *
     * @param i level
     */
    public void openLevel(int i) {
        if (i > 1 && i <= getNumberOfLevels()) {
            levelOpen[i - 1] = true;
            if (i > settings.getOpenLevels()) {
                settings.setOpenLevels(i);
            }
        }
    }

    /**
     * returns number of total levels.
     *
     * @return number of levels
     */
    public int getNumberOfLevels() {
        return levelOpen.length;
    }
}
