/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller.menu;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Polygon;
import org.newdawn.slick.state.StateBasedGame;

/**
 * A <code>AbstractMenuItem</code> represents one part of a <code>Menu</code>.
 * It contains a visual representation that is shown to represent it when
 * the menu it belongs to is shown. It also contains a <code>MenuAction</code>
 * which contains instruction on how to act if isMouseOverItem returns
 * true and the user pressed a mouse button.
 *
 * @see Menu
 * @see MenuAction
 *
 * @author sam
 */
public abstract class AbstractMenuItem {

    /**
     * the scale at which the image is shown (size).
     * Will be between 1 and 1.05 and depends on position of mouse.
     */
    protected float scale = 1;
    /** whether or not this item is selected (used for keyboard input). */
    protected boolean selected = false;
    /** a polygon surrounding the picture. */
    private Polygon polygon;
    /** the amount the scale is change if Item is (de-)selected. */
    private static final float SCALESTEP = 0.0001f;
    /** the top left x position of this image.*/
    protected float xPos;
    /** the top left y position of this image.*/
    protected float yPos;
    /** the action to execute. */
    protected MenuAction action;

    /**
     * constructs new AbstractMenuItem.
     *
     * @param xPos xPos
     * @param yPos yPos
     * @param action action to execute
     */
    public AbstractMenuItem(float xPos, float yPos, MenuAction action) {
        this.xPos = xPos;
        this.yPos = yPos;
        this.action = action;
    }

    /**
     * returns whether or not this item is active.
     *
     * @return is active?
     */
    public boolean isActive() {
        return action.isActivated();
    }

    /**
     * returns true, if the position given by x and y is inside the polygon
     * surrounding this MenuItem.
     *
     * @param x x position of mouse
     * @param y y position of mouse
     * @return true if mouse is over MenuItem, false otherwise
     */
    private boolean isMouseOverItem(float x, float y) {
        return getPolygon().contains(x, y);
    }

    /**
     * Checks weather or not this MenuItem is affected by the
     * current condition of the mouse.
     * If the mouse is positioned on it, the image representing it
     * will slowly grow
     * to indicate that this item is currently selected.
     * If the mouse is not positioned on it and it is still enlarged,
     * it will shrink.
     * If the mouse is positioned on it, its MenuAction will be executed.
     *
     * @param x position of mouse on x axis
     * @param y position of mouse on y axis
     * @param mouseButtonDown is mouse clicked?
     * @param i delta value
     * @param gc gamecontainer
     * @param sbg statebased game
     * @throws SlickException SlickException
     */
    public void applyMouseInput(float x, float y, boolean mouseButtonDown,
            int i, GameContainer gc, StateBasedGame sbg)
            throws SlickException {
        if (isMouseOverItem(x, y) && action.isActivated()) {
            if (scale < 1.05f) {
                scale += SCALESTEP * i;
            }
            if (mouseButtonDown && action.isActivated()) {
                action.execute(gc, sbg, this);
            }
        } else if (!selected) {
            if (scale > 1.0f) {
                scale -= SCALESTEP * i;
            }
        }
    }

    /**
     * creates a square.
     * <p>
     * The square should be around the visual representation of the MenuItem.
     * It will be used to detect if the mouse if it or not.
     * <p>
     * Must be called in sub-class constructor!
     *
     * @param x top x position
     * @param y top y position
     * @param width width
     * @param higth higth
     * @return
     */
    protected void generateSquare(float x, float y, float width, float higth) {
        polygon = new Polygon(new float[]{
                    x, y,
                    x, y + higth,
                    x + width, y + higth,
                    x + width, y});
    }

    /**
     * renders this menu to the screen.
     *
     * @param grphcs graphics context
     */
    public abstract void render(Graphics grphcs);

    /**
     * @return the polygon
     */
    public Polygon getPolygon() {
        return polygon;
    }

    /* Methods for keyboard input: */

    /**
     * is this item currently selected?
     *
     * @return is selected?
     */
    public boolean isSelected() {
        return selected;
    }

    /**
     * select this item.
     */
    public void select() {
        selected = true;
        scale = 1.05f;
    }

    /**
     * unselect this item.
     */
    public void unSelect() {
        selected = false;
        scale = 1.0f;
    }

    /**
     * execute the action associated with this item.
     *
     * @param gc GameContainer
     * @param sbg StateBasedGame
     * @throws SlickException SlickException
     */
    public void execute(GameContainer gc, StateBasedGame sbg)
            throws SlickException {
        action.execute(gc, sbg, this);
    }
}
