/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package sidescroller.menu;

import java.util.LinkedList;
import java.util.List;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.UnicodeFont;
import org.newdawn.slick.state.StateBasedGame;

/**
 * This class represents a Menu.
 * In this case, Menu means one container containing one or more MenuItems,
 * which themselves can link to a different Menu or be responsible for a setting
 * or something similar.
 *
 * @see MenuItem
 *
 * @author sam
 */
public class Menu {

    private List<AbstractMenuItem> items;
    private float centerXPosition;
    private float topYPosition;
    private float yspaceBetweenItems;
    private int oldMouseX;
    private int oldMouseY;

    /**
     * constructs a new Menu.
     *
     * @param centerXPosition x position the center of this
     * menu should be rendered at
     * @param topYPosition y position the top of this
     * menu should be rendered at
     * @param yspaceBetweenItems space between MenuItems on y axis
     */
    public Menu(float centerXPosition, float topYPosition,
            float yspaceBetweenItems) {
        this.centerXPosition = centerXPosition;
        this.topYPosition = topYPosition;
        this.yspaceBetweenItems = yspaceBetweenItems;
        items = new LinkedList<AbstractMenuItem>();
    }

    /**
     * adds a new MenuItemImage to this menu.
     *
     * @see MenuItemImage
     *
     * @param img the image to represent this item
     * @param action the action to be executed by this item
     */
    public void addItem(Image img, MenuAction action) {
        MenuItemImage newItem = new MenuItemImage(img,
                centerXPosition,
                (topYPosition + (items.size() * yspaceBetweenItems)),
                action);
        if (items.isEmpty()) {
            newItem.select();
        }
        items.add(newItem);
    }

    /**
     * adds a new MenuItemString to this menu.
     *
     * @see MenuItemString
     *
     * @param string string to display
     * @param fond fond to use
     * @param action action to execute
     */
    public void addItem(String string, UnicodeFont fond, MenuAction action) {
        MenuItemString newItem = new MenuItemString(string, fond,
                centerXPosition,
                (topYPosition + (items.size() * yspaceBetweenItems)),
                action);
        if (items.isEmpty()) {
            newItem.select();
        }
        items.add(newItem);
    }

    /**
     * let this menu or its MenuItems react to userinput/changed conditions.
     *
     * @param gc GameContainer
     * @param sbg StateBasedGame
     * @param i delta
     * @throws SlickException SlickException
     */
    public void update(GameContainer gc, StateBasedGame sbg, int i)
            throws SlickException {
        // FIXME mouse and keyboard input work well for themselves but do not
        // work well together (if mouse is used, the keyboard selection will
        // stay bright white (selected)).
        Input input = gc.getInput();

        int mouseX = input.getMouseX();
        int mouseY = input.getMouseY();
        boolean mouseClicked = input.isMousePressed(Input.MOUSE_LEFT_BUTTON);

        boolean downKeyPressed = input.isKeyPressed(Input.KEY_DOWN);
        boolean upKeyPressed = input.isKeyPressed(Input.KEY_UP);
        boolean enterKeyPressed = input.isKeyPressed(Input.KEY_ENTER);

        for (int j = 0; j < items.size(); j++) {
            AbstractMenuItem menuItem = items.get(j);
            // only apply mouse input if mouse moved or was clicked
            // (otherwise keyboard input does not work)
            if ((mouseX != oldMouseX && mouseY != oldMouseY) || mouseClicked) {
                menuItem.applyMouseInput(mouseX, mouseY,
                        mouseClicked, i, gc, sbg);
            }

            // keyboard input:
            if (downKeyPressed) {
                if (menuItem.isSelected()) {
                    menuItem.unSelect();
                    selectNext(j);
                    return;
                }
            } else if (upKeyPressed) {
                if (menuItem.isSelected()) {
                    menuItem.unSelect();
                    selectPrevious(j);
                    return;
                }
            } else if (enterKeyPressed) {
                if (menuItem.isSelected()) {
                    menuItem.execute(gc, sbg);
                }
            }
        }
    }

    /**
     * select the previous stored MenuItem before i. If it is not active,
     * the one before that will be selected.
     *
     * @param i current
     */
    private void selectPrevious(int i) {
        if (i > 0) {
            if (!items.get(i - 1).isActive()) {
                selectPrevious(i - 1);
            } else {
                items.get(i - 1).select();
            }
        } else {
            if (!items.get(items.size() - 1).isActive()) {
                selectNext(items.size() - 2);
            } else {
                items.get(items.size() - 1).select();
            }
        }
    }

    /**
     * select the next stored MenuItem after i. If it is not active,
     * the one after that will be selected.
     *
     * @param i current
     */
    private void selectNext(int i) {
        if (i < items.size() - 1) {
            if (!items.get(i + 1).isActive()) {
                selectNext(i + 1);
            } else {
                items.get(i + 1).select();
            }
        } else {
            if (!items.get(0).isActive()) {
                selectNext(0);
            } else {
                items.get(0).select();
            }
        }
    }

    /**
     * draws this menu/its MenuItems to the screen.
     *
     * @param grphcs graphics context
     */
    public void render(Graphics grphcs) {
        for (int i = 0; i < items.size(); i++) {
            AbstractMenuItem menuItem = items.get(i);
            menuItem.render(grphcs);
            if (sidescroller.Settings.debug) {
                grphcs.draw(menuItem.getPolygon());
            }
        }
    }
}
